(function() {
  var Query, Serializer, each, map;

  map = require('lodash/map');

  each = require('lodash/each');

  Serializer = require('./serializer');

  Query = (function() {
    var ops;

    class Query {
      constructor(q = [], serializer = null) {
        this.serializer = serializer != null ? serializer : new Serializer();
        this.q = q;
        return;
      }

      prepare(col_ref, serializer = null) {
        var fbq, s;
        s = serializer != null ? serializer : this.serializer;
        fbq = col_ref;
        each(this.q, function(statement) {
          fbq = ops[statement[0]](fbq, s, ...statement.slice(1));
        });
        return fbq;
      }

    };

    Query.prototype.type = 'Query';

    // firestore query (fbq), serializer (s), fields (f), values (v)
    ops = {
      'eq': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '==', v);
      },
      'eq-any': function(fbq, s, f, vs) {
        return fbq.where(s.fmt[s.fmts.field.db](f), 'in', vs);
      },
      'gt': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '>', v);
      },
      'gte': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '>=', v);
      },
      'includes': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), 'array-contains', v);
      },
      'includes-any': function(fbq, s, f, vs) {
        return fbq.where(s.fmt[s.fmts.field.db](f), 'array-contains-any', vs);
      },
      'limit': function(fbq, _, n) {
        return fbq.limit(n);
      },
      'lt': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '<', v);
      },
      'lte': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '<=', v);
      },
      'not-eq': function(fbq, s, f, v) {
        return fbq.where(s.fmt[s.fmts.field.db](f), '!=', v);
      },
      'not-eq-any': function(fbq, s, f, vs) {
        return fbq.where(s.fmt[s.fmts.field.db](f), 'not-in', vs);
      },
      'order-by': function(fbq, s, f, d) {
        return fbq.orderBy(s.fmt[s.fmts.field.db](f), d != null ? d : 'asc');
      },
      'start-at': function(fbq, _, v) {
        return fbq.startAt(...v);
      }
    };

    return Query;

  }).call(this);

  module.exports = Query;

}).call(this);
